package cz.zweistein.gw.predictor.controller;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import cz.zweistein.gw.predictor.model.Guild;
import cz.zweistein.gw.predictor.model.Player;
import cz.zweistein.gw.predictor.model.User;
import cz.zweistein.gw.predictor.util.ListPrintoutUtil;

/**
 * 
 * Remote client for Guild Wars Xunlai Tournament House website 
 * 
 * @author zwei
 *
 */
public class XTHRemote {
	
	public XTHRemote() {
		// nil
	}

	private static Logger logger = Logger.getLogger(XTHRemote.class);

	private static int NUMBER_OF_VOTABLE_ITEMS = 8;

	private HttpClient httpClient = new DefaultHttpClient();

	private boolean loggedIn;
	private String vote1v1Token = null;
	private String voteGvGToken = null;

	public boolean isLoggedIn() {
		return this.loggedIn;
	}
	

	public void logIn(User user) {
		this.loggedIn = false;

		logger.info(user.getEmail() + " logging in.");
		
		logger.debug("Obtaining loging token");
		String loginToken = null;
		
		HttpPost tokenPost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/");
		
		try {
			HttpResponse tokenResponse = this.httpClient.execute(tokenPost);
			HttpEntity tokeEntity = tokenResponse.getEntity();
			String tokenPageContent = EntityUtils.toString(tokeEntity);
			
			String tokenRegExp = "<input type=\"hidden\" name=\"loginToken\" value=\"([a-z0-9]+)\" />";
			
			Pattern pattern = Pattern.compile(tokenRegExp);
			Matcher matcher = pattern.matcher(tokenPageContent);

			if (matcher.find()) {
				loginToken = matcher.group(1);
				logger.debug("Login token: " + loginToken);
			} else {
				logger.debug("Login token not found.");
			}
			
		} catch (ClientProtocolException e1) {
			e1.printStackTrace();
		} catch (ParseException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (loginToken == null) {
				logger.error("Token not obtained.");
			}
		}

		HttpPost httpPost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/default.php");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();

		nvps.add(new BasicNameValuePair("username", user.getEmail()));
		nvps.add(new BasicNameValuePair("password", user.getPassword()));
		nvps.add(new BasicNameValuePair("execute", "submit"));
		nvps.add(new BasicNameValuePair("submit.x", "0"));
		nvps.add(new BasicNameValuePair("submit.y", "0"));
		nvps.add(new BasicNameValuePair("loginToken", loginToken));

		try {
			httpPost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

			HttpResponse response = this.httpClient.execute(httpPost);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity);
			
			// get cookie?

			if (pageContent.contains("(Code=w.104)")) {
				logger.info("Failed to log in");
				// failure to log-in.
				this.loggedIn = false;
			} else {
				logger.info("Logged in.");

				this.loggedIn = true;

				/*String userIdPattern = "<input type=\"hidden\" name=\"user_id\" value=\"([0-9]*)\" />";
				
				Pattern pattern = Pattern.compile(userIdPattern);
				Matcher matcher = pattern.matcher(pageContent);

				if (matcher.find()) {
					String userId = matcher.group(1);
					this.userId = userId;
				}*/

			}

		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	public List<Guild> getQualifiedGuilds() {
		logger.info("Getting information about which guilds are qualified.");

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/xth.php");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps.add(new BasicNameValuePair("continue", "View GvG Ballot"));
		nvps.add(new BasicNameValuePair("Submit", "submit"));
		nvps.add(new BasicNameValuePair("submit.x", "0"));
		nvps.add(new BasicNameValuePair("submit.y", "0"));

		List<Guild> result = new LinkedList<Guild>();

		try {
			httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
			HttpResponse response = this.httpClient.execute(httpost);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity, "UTF-8");
			
			String regEx = "<div class=\"itemNameStyle\">([\\p{Alpha}\\p{InHiragana}\\p{InKatakana}\\p{InHangulJamo}\\p{InHangulSyllables} ]+)\\[([\\p{Alpha}\\p{InHiragana}\\p{InKatakana}\\p{InHangulJamo}\\p{InHangulSyllables}]+)\\]</div><div class=\"QPointStyle\">\\(Q.P. ([0-9]+)\\)</div><input type=\"hidden\" name=\"itemId\" value=\"([0-9]+)\" />";

			Pattern pattern = Pattern.compile(regEx);
			Matcher matcher = pattern.matcher(pageContent);

			while (matcher.find()) {
				Guild guild = new Guild();
				guild.setId(matcher.group(4));
				guild.setName(matcher.group(1).trim());
				guild.setTag(matcher.group(2).trim());
				guild.setQualifierPoints(Integer.parseInt(matcher.group(3)));
				result.add(guild);
			}
			
			logger.info(result.size() + " guilds qualify.");
			
			// also, fill in token for voting, important!
			String tokenRegEx = "<input type=\"hidden\" name=\"transToken\" value=\"([a-z0-9]+)\" />";
			
			Pattern tokenPattern = Pattern.compile(tokenRegEx);
			Matcher tokenMatcher = tokenPattern.matcher(pageContent);
			
			if (tokenMatcher.find()) {
				this.voteGvGToken = tokenMatcher.group(1);
				logger.debug("GvG Token: "+this.voteGvGToken);
			} else {
				logger.info("Failed to obtain token to vote for GvG");
			}

		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return result;
	}

	public int getGuildLadderRating(String guildName) {

		logger.info("Getting ladder rating for guild " + guildName);

		String fixedGuildName = guildName.replaceAll(" ", "%20");

		String url = "https://www.guildwars.com/competitive/ladder/get_ladder.php?batch=null&column=null&order=null&from=0&max=null&name="
				+ fixedGuildName + "&tag=null&lang=en";

		String ratingPattern = "<td><div id=\"opaquetext\">&nbsp;([0-9]+)</div>";

		String notFound = "None found.";

		int rating = 0;

		try {
			HttpGet httpget = new HttpGet(url);
			HttpResponse response = this.httpClient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity);

			if (pageContent.indexOf(notFound) > 0) {
				return 0;
			} else {
				Pattern pattern = Pattern.compile(ratingPattern);
				Matcher matcher = pattern.matcher(pageContent);

				if (matcher.find()) {
					rating = Integer.parseInt(matcher.group(1));
				}
			}
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return rating;
	}

	public void fillInPlayerTournamentHistory(List<Player> players) {

		Calendar calNow = new GregorianCalendar();
		Calendar calTournament = new GregorianCalendar(2008, 0, 1);
		
		Calendar calURLChange = new GregorianCalendar(2008, 11, 1);

		while (calTournament.getTimeInMillis() < calNow.getTimeInMillis()) {

			String url = "http://www.guildwars.com/competitive/tournament"
					+ calTournament.get(Calendar.YEAR)
					+ "/"
					+ calTournament.getDisplayName(Calendar.MONTH,
							Calendar.LONG, Locale.US).toLowerCase()
					+ "/1v1results"
					+ (calURLChange.getTimeInMillis()>calTournament.getTimeInMillis() ? "elim" : "")
					+ ".php";

			logger.info("Loading: " + url);

			try {

				HttpGet httpget = new HttpGet(url);
				HttpResponse response = this.httpClient.execute(httpget);

				HttpEntity entity = response.getEntity();
				String pageContent = EntityUtils.toString(entity, "UTF-8");
				if (pageContent
						.indexOf("We're sorry, the page you requested could not be found.") > 0) {
					logger.debug("Loading failed.");
					break;
				}
				String resultTable = pageContent.substring(pageContent
						.indexOf("Elimination Results"), pageContent
						.indexOf("Swiss Rounds Results"));

				String regEx;
				// historical page change in december 2008
				if (calURLChange.getTimeInMillis()>calTournament.getTimeInMillis()) {
					regEx = "<td>(.+)</td><td>(.+)</td>";
				} else {
					regEx = "<td>(.+)</td><td><a href='/competitive/hero_ladder/show_hero.php\\?guid=(.+)\'>(.+)</a></td>";
				}

				Pattern pattern = Pattern.compile(regEx);
				Matcher matcher = pattern.matcher(resultTable);

				while (matcher.find()) {
					int order = Integer.parseInt(matcher.group(1));
					String playerName = null;
					String guid = null;
					if (calURLChange.getTimeInMillis()>calTournament.getTimeInMillis()) {
						playerName = matcher.group(2).trim();
					} else {
						guid = matcher.group(2).trim();
						playerName = matcher.group(3).trim();
					}
					
					logger.debug("Player " + playerName + " parsed as " + order);

					for (Player player : players) {
						if (player.getName().equals(playerName)) {
							player.addOrder(order);
							player.setGuid(guid);
							break;
						}
					}
				}

				calTournament.add(Calendar.MONTH, 1);

			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

	}

	public void fillInGuildTournamentHistory(List<Guild> guilds) {

		Calendar calNow = new GregorianCalendar();
		Calendar calTournament = new GregorianCalendar(2008, 0, 1);
		
		Calendar calURLChange = new GregorianCalendar(2008, 11, 1);
		
		while (calTournament.getTimeInMillis() < calNow.getTimeInMillis()) {

			String url = "http://www.guildwars.com/competitive/tournament"
					+ calTournament.get(Calendar.YEAR)
					+ "/"
					+ calTournament.getDisplayName(Calendar.MONTH,
							Calendar.LONG, Locale.US).toLowerCase()
					+ "/gvgresults"
					+ (calURLChange.getTimeInMillis()>calTournament.getTimeInMillis() ? "elim" : "")
					+ ".php";

			logger.info("Attempting to load " + url);

			try {
				HttpGet httpget = new HttpGet(url);
				HttpResponse response = this.httpClient.execute(httpget);

				HttpEntity entity = response.getEntity();
				String pageContent = EntityUtils.toString(entity, "UTF-8");
				if (pageContent
						.indexOf("We're sorry, the page you requested could not be found.") > 0) {
					logger.debug("Page load failed.");
					break;
				}
				String resultTable = pageContent.substring(pageContent
						.indexOf("Elimination Results"), pageContent
						.indexOf("Swiss Rounds Results"));
				
				String regEx = "";
				
				if (calURLChange.getTimeInMillis()>calTournament.getTimeInMillis()) {
					regEx = "<td>(.+)</td><td>(.+)</td>";
				} else {
					regEx = "<td>(.+)</td><td><a href='/competitive/ladder/show_guild.php\\?guid=(.+)\'>(.+)</a></td>";
				}

				Pattern pattern = Pattern.compile(regEx);
				Matcher matcher = pattern.matcher(resultTable);

				while (matcher.find()) {
					int order = Integer.parseInt(matcher.group(1));
					String guildName = null;
					String guid = null;
					if (calURLChange.getTimeInMillis()>calTournament.getTimeInMillis()) {
						guildName = matcher.group(2).trim();
					} else {
						guid = matcher.group(2).trim();
						guildName = matcher.group(3).trim();
					}
					
					logger.debug("Guild " + guildName + " parsed as " + order);

					for (Guild guild : guilds) {
						if (guild.getName().equals(guildName)) {
							guild.addOrder(order);
							guild.setGuid(guid);
							break;
						}
					}
				}

				calTournament.add(Calendar.MONTH, 1);

			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

	}

	public void voteForGvG(List<Guild> candidates)
			throws ClientProtocolException, IOException {

		if (candidates.size() != NUMBER_OF_VOTABLE_ITEMS) {
			logger.error("Unable to vote as " + candidates.size()
					+ " guild names were suppiled instead of " + NUMBER_OF_VOTABLE_ITEMS + ". Aborting voting for GvG.");
			return;
		}

		StringBuilder nodeIdsCSV = new StringBuilder();
		for (int i = 0; i < NUMBER_OF_VOTABLE_ITEMS; i++) {
			
			if (candidates.get(i) == null) {
				logger.error("Guild #" + (i+1) + " was not recognized. Aborting voting for GvG." );
				return;
			}
			
			nodeIdsCSV.append(candidates.get(i).getId());
			nodeIdsCSV.append(",");
		}

		logger.info("Voting For GvG: " + ListPrintoutUtil.list2Print(candidates));

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/xth.php");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps
				.add(new BasicNameValuePair("hiddenNodeIds", nodeIdsCSV
						.toString()));

		nvps.add(new BasicNameValuePair("vote_ballot", "GvG"));
		nvps.add(new BasicNameValuePair("continue", "Save Predictions"));
		nvps.add(new BasicNameValuePair("submit.x", "0"));
		nvps.add(new BasicNameValuePair("submit.y", "0"));
		nvps.add(new BasicNameValuePair("submit", "Submit"));
		nvps.add(new BasicNameValuePair("transToken", this.voteGvGToken));

		httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

		HttpResponse response = this.httpClient.execute(httpost);
		HttpEntity entity = response.getEntity();
		entity.consumeContent();
	}

	public List<Player> getQualifiedPlayers() {
		logger.info("Getting information about which players are qualified.");

		HttpPost httpPost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/xth.php");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps.add(new BasicNameValuePair("continue", "View 1v1 Ballot"));
		nvps.add(new BasicNameValuePair("action", "insert"));
		nvps.add(new BasicNameValuePair("Submit", "submit"));
		nvps.add(new BasicNameValuePair("submit.x", "0"));
		nvps.add(new BasicNameValuePair("submit.y", "0"));

		List<Player> result = new LinkedList<Player>();

		try {
			httpPost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
			HttpResponse response = this.httpClient.execute(httpPost);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity, "UTF-8");

			String regEx = "<div class=\"itemNameStyle\">([\\p{Alpha}\\p{InHiragana}\\p{InKatakana}\\p{InHangulJamo}\\p{InHangulSyllables} ]+)</div><div class=\"QPointStyle\">\\(Q.P. ([0-9]*)\\)</div><input type=\"hidden\" name=\"itemId\" value=\"(\\d+)\" />";

			Pattern pattern = Pattern.compile(regEx);
			Matcher matcher = pattern.matcher(pageContent);

			while (matcher.find()) {
				Player player = new Player();
				player.setId(matcher.group(3));
				player.setName(matcher.group(1).trim());
				player.setQualifierPoints(Integer.parseInt(matcher.group(2)));
				result.add(player);
			}
			
			logger.debug(result.size() + " players qualify.");
			
			// also, fill in token for voting, important!
			String tokenRegEx = "<input type=\"hidden\" name=\"transToken\" value=\"([a-z0-9]+)\" />";
			
			Pattern tokenPattern = Pattern.compile(tokenRegEx);
			Matcher tokenMatcher = tokenPattern.matcher(pageContent);
			
			if (tokenMatcher.find()) {
				this.vote1v1Token = tokenMatcher.group(1);
				logger.debug("1v1 Token: "+this.vote1v1Token);
			} else {
				logger.info("Failed to obtain token to vote for 1v1.");
			}

		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return result;
	}

	public void voteFor1v1(List<Player> candidates)
			throws ClientProtocolException, IOException {

		if (candidates.size() < NUMBER_OF_VOTABLE_ITEMS) {
			logger.error("Unable to vote as " + candidates.size()
					+ " player names were suppiled instead of " + NUMBER_OF_VOTABLE_ITEMS + ". Aborting voting for 1v1.");
			return;
		}

		StringBuilder nodeIdsCSV = new StringBuilder();
		for (int i = 0; i < NUMBER_OF_VOTABLE_ITEMS; i++) {
			
			if (candidates.get(i) == null) {
				logger.error("Player #" + (i+1) + " was not recognized. Aborting voting for 1v1." );
				return;
			}
			
			nodeIdsCSV.append(candidates.get(i).getId());
			nodeIdsCSV.append(",");
		}

		logger.info("Voting For 1v1: " + ListPrintoutUtil.list2Print(candidates));

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/xth.php");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps
				.add(new BasicNameValuePair("hiddenNodeIds", nodeIdsCSV
						.toString()));
		
		nvps.add(new BasicNameValuePair("vote_ballot", "1v1"));
		nvps.add(new BasicNameValuePair("continue", "Save Predictions"));
		nvps.add(new BasicNameValuePair("submit.x", "0"));
		nvps.add(new BasicNameValuePair("submit.y", "0"));
		nvps.add(new BasicNameValuePair("submit", "Submit"));
		nvps.add(new BasicNameValuePair("transToken", this.voteGvGToken));

		httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

		HttpResponse response = this.httpClient.execute(httpost);
		HttpEntity entity = response.getEntity();
		entity.consumeContent();
	}

}
